unit Main_U;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TMain_F = class(TForm)
    pbSet: TButton;
    pbClose: TButton;
    GroupBox1: TGroupBox;
    rbDelphi4: TRadioButton;
    rbDelphi5: TRadioButton;
    pcPathsDisplay: TPageControl;
    tsLibraryPath: TTabSheet;
    reLibraryPath: TRichEdit;
    tsBrowsingPath: TTabSheet;
    reBrowsingPath: TRichEdit;
    rbDelphi6: TRadioButton;
    rbDelphi7: TRadioButton;
    rbDelphi2007: TRadioButton;
    pbAbout: TButton;
    rbDelphi2006: TRadioButton;
    procedure pbSetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbCloseClick(Sender: TObject);
    procedure rbDelphiClick(Sender: TObject);
    procedure pbAboutClick(Sender: TObject);
  private
    procedure RefreshLibPathDisplay();
    function  GetLibKey(): string;
    procedure ForceEnvOptionsUpdate();
  public
    { Public declarations }
  end;

var
  Main_F: TMain_F;

implementation

{$R *.DFM}

uses
  registry,
  About_U,
  SDUGeneral,
  SDUDialogs;


const
  LIBPATH_REG_KEY_4_TO_7        = 'Borland\Delphi\%s\Library';
  LIBPATH_REG_KEY_2007_LIB      = 'Borland\BDS\%s\Library';
  LIBPATH_REG_KEY_2007_GLOBALS  = 'Borland\BDS\%s\Globals';
  LIBPATH_REG_ENTRY_LIBRARY_PATH  = 'Search Path';
  LIBPATH_REG_ENTRY_BROWSING_PATH = 'Browsing Path';
  LIBPATH_REG_ENTRY_FORCEENVOPTIONSUPDATE = 'ForceEnvOptionsUpdate';


// Save user's lib paths
procedure TMain_F.pbSetClick(Sender: TObject);
var
  libraryPath: string;
  browsingPath: string;
  libraryRegKey: string;
  i: integer;
  regIni: TRegIniFile;
begin
  libraryRegKey := GetLibKey();

  // Library path...
  libraryPath := '';
  for i:=0 to (reLibraryPath.lines.count-1) do
    begin
    if i>0 then
      begin
      libraryPath := libraryPath + ';';
      end;
    libraryPath := libraryPath + reLibraryPath.lines[i];
    end;

  // Browsing path...
  browsingPath := '';
  for i:=0 to (reBrowsingPath.lines.count-1) do
    begin
    if i>0 then
      begin
      browsingPath := browsingPath + ';';
      end;
    browsingPath := browsingPath + reBrowsingPath.lines[i];
    end;


  regIni := TRegIniFile.Create('Software');
  try
    regIni.WriteString(libraryRegKey, LIBPATH_REG_ENTRY_LIBRARY_PATH,  libraryPath);
    regIni.WriteString(libraryRegKey, LIBPATH_REG_ENTRY_BROWSING_PATH, browsingPath);
  finally
    regIni.Free();
  end;

  ForceEnvOptionsUpdate();

  SDUMessageDlg(
                'Paths updated'+SDUCRLF+
                SDUCRLF+
                'Please restart Delphi for changes to take effect.',
                mtInformation,
                [mbOK],
                0
               );

end;


procedure TMain_F.FormCreate(Sender: TObject);
begin
  pcPathsDisplay.ActivePage := tsLibraryPath;
  rbDelphi2007.Checked := TRUE;
  RefreshLibPathDisplay();

end;

// Load in lib paths and update display with them
procedure TMain_F.RefreshLibPathDisplay();
var
  libraryPath: string;
  browsingPath: string;
  currPath: string;
  libraryRegKey: string;
  regIni: TRegIniFile;
begin
  libraryRegKey := GetLibKey();

  regIni := TRegIniFile.Create('Software');
  try
    libraryPath  := regIni.ReadString(libraryRegKey, LIBPATH_REG_ENTRY_LIBRARY_PATH, '');
    browsingPath := regIni.ReadString(libraryRegKey, LIBPATH_REG_ENTRY_BROWSING_PATH, '');
  finally
    regIni.Free();
  end;


  // Library path...
  reLibraryPath.lines.clear();
  while (pos(';', libraryPath)>0) do
    begin
    currPath := libraryPath;
    delete(currPath, pos(';', currPath), length(currPath)-pos(';', currPath)+1);
    delete(libraryPath, 1, pos(';', libraryPath));

    reLibraryPath.lines.add(currPath);
    end;
  reLibraryPath.lines.add(libraryPath);

  
  // Browsing path...
  reBrowsingPath.lines.clear();
  while (pos(';', browsingPath)>0) do
    begin
    currPath := browsingPath;
    delete(currPath, pos(';', currPath), length(currPath)-pos(';', currPath)+1);
    delete(browsingPath, 1, pos(';', browsingPath));

    reBrowsingPath.lines.add(currPath);
    end;
  reBrowsingPath.lines.add(browsingPath);

end;

procedure TMain_F.pbAboutClick(Sender: TObject);
begin
  ShowAboutDialog();
end;

procedure TMain_F.pbCloseClick(Sender: TObject);
begin
  Close;

end;

procedure TMain_F.rbDelphiClick(Sender: TObject);
begin
  RefreshLibPathDisplay();

end;

function TMain_F.GetLibKey(): string;
var
  retval: string;
  versionStr: string;
begin
  if(
     rbDelphi4.checked or
     rbDelphi5.checked or
     rbDelphi6.checked or
     rbDelphi7.checked
    ) then
    begin
    if rbDelphi4.checked then
      begin
      versionStr := '4.0';
      end
    else if rbDelphi5.checked then
      begin
      versionStr := '4.0';
      end
    else if rbDelphi6.checked then
      begin
      versionStr := '6.0';
      end
    else if rbDelphi7.checked then
      begin
      versionStr := '7.0';
      end;

    retval := Format(LIBPATH_REG_KEY_4_TO_7, [versionStr]);
    end
  else if (
           rbDelphi2007.checked or
           rbDelphi2007.checked
          ) then
    begin
    if rbDelphi2006.checked then
      begin
      versionStr := '4.0';
      end
    else if rbDelphi2007.checked then
      begin
      versionStr := '5.0';
      end;

    retval := Format(LIBPATH_REG_KEY_2007_LIB, [versionStr]);
    end;


  Result := retval;
end;

// Note: This will only set the flag reg key if it's needed
// From the Delphi 2007 README, this must be set to "1" in order for Delphi to
// automatically update "EnvOptions.proj" when Delphi next starts.
// See:
// http://dn.codegear.com/article/36575
// Release Notes for Delphi 2007 for Win32 and C++Builder 2007
procedure TMain_F.ForceEnvOptionsUpdate();
var
  globalsKey: string;
  regIni: TRegIniFile;
begin
  if (
      rbDelphi2006.checked or  // Not sure if this is needed for Delphi 2006?
      rbDelphi2007.checked
     ) then
    begin
    if rbDelphi2006.checked then
      begin
      globalsKey := Format(LIBPATH_REG_KEY_2007_GLOBALS, ['4.0']);
      end
    else
      begin
      globalsKey := Format(LIBPATH_REG_KEY_2007_GLOBALS, ['5.0']);
      end;

    regIni := TRegIniFile.Create('Software');
    try
      regIni.WriteString(globalsKey, LIBPATH_REG_ENTRY_FORCEENVOPTIONSUPDATE, '1');
    finally
      regIni.Free();
    end;

    end;

end;

END.

